/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.otherspecies;

import java.util.HashMap;
import java.util.regex.Matcher;

import eu.omp.irap.cassis.common.CassisDecimalFormat;
import eu.omp.irap.cassis.common.LineDescription;
import eu.omp.irap.cassis.common.axes.UNIT;
import eu.omp.irap.cassis.common.axes.XAxisCassis;
import eu.omp.irap.cassis.database.access.MoleculeDescriptionDB;
import eu.omp.irap.cassis.database.access.VamdcDataBaseConnection;
import eu.omp.irap.cassis.properties.Software;

/**
 * Class to store constants about Other Species labels.
 *
 * @author M. Boiziot
 */
public class SpeciesLabelConstants {

	public static final String AIJ_UNIT = "aijUnit";
	public static final String AIJ_VALUE = "aijValue";

	public static final String CITATION_LABEL = "citationLabel";
	public static final String CITATION_VALUE = "citationValue";

	public static final String ELOW_UNIT = "elowUnit";
	public static final String ELOW_VALUE = "elowValue";

	public static final String EUP_UNIT = "eupUnit";
	public static final String EUP_VALUE = "eupValue";

	public static final String GUP_UNIT = "gupUnit";
	public static final String GUP_VALUE = "gupValue";

	public static final String MOL_NAME_VALUE = "molNameValue";

	public static final String OTHER_WAVE = "otherWave";

	public static final String QUANTIC_NUMBER_VALUE = "quanticNumberValue";

	public static final String SHIFT_WAVE_LABEL = "shiftWaveLabel";
	public static final String SHIFT_WAVE_UNIT = "shiftWaveUnit";
	public static final String SHIFT_WAVE_VALUE = "shiftWaveValue";

	public static final String TAG_VALUE = "tagValue";

	public static final String WAVE_ERROR_LABEL_BEGIN = "waveErrorLabelBegin";
	public static final String WAVE_ERROR_LABEL_END = "waveErrorLabelEnd";
	public static final String WAVE_ERROR_VALUE = "waveErrorValue";

	public static final String WAVE_LABEL = "waveLabel";
	public static final String WAVE_UNIT = "waveUnit";
	public static final String WAVE_VALUE = "waveValue";


	/**
	 * Create the mapping between the template otherSpecies line identification
	 * and the parameters of the line.
	 *
	 * @param line The line
	 * @param wave The wave
	 * @param waveError The wave error
	 * @param energyUnit The energy unit
	 * @param vlsrPlot The VLSR of the plot
	 * @param xAxisCassis The XAxisCassis used
	 * @return the mapping
	 */
	private static HashMap<String, String> getParameterValueMapping(
			LineDescription line, double wave, double waveError,
			UNIT energyUnit, double vlsrPlot, XAxisCassis xAxisCassis) {
		HashMap<String, String> map = new HashMap<String, String>();

		map.put(MOL_NAME_VALUE, line.getMolName());
		map.put(QUANTIC_NUMBER_VALUE, MoleculeDescriptionDB.constitueLine(line.getQuanticN()));
		map.put(TAG_VALUE, String.valueOf(line.getSpeciesId()));
		map.put(WAVE_LABEL, xAxisCassis.getAxis().getSymbol()+" =");
		map.put(WAVE_VALUE, CassisDecimalFormat.FREQUENCY_FORMAT.format(wave));
		map.put(WAVE_ERROR_LABEL_BEGIN, "(±");
		map.put(WAVE_ERROR_VALUE, CassisDecimalFormat.FREQUENCY_FORMAT.format(waveError));
		map.put(WAVE_UNIT, xAxisCassis.getUnit().getValString());
		map.put(WAVE_ERROR_LABEL_END, ") ");
		if (line.getComment()!= null && !line.getComment().isEmpty()) {
			map.put("waveComment", line.getComment());
		}
		double elowValue = UNIT.convertEnergyLevel(line.getElowCminus1(),
				UNIT.CM_MINUS_1, energyUnit);
		String elowValueString = CassisDecimalFormat.EUP_FORMAT.format(elowValue);
		if (elowValue < 10E-1) {
			elowValueString = CassisDecimalFormat.TWO_DEC_EXP_FORMAT.format(elowValue);
		}
		map.put(ELOW_VALUE, elowValueString);
		map.put(ELOW_UNIT, energyUnit.getValString());

		double eupValue = UNIT.convertEnergyLevel(line.getEUpK(),
				UNIT.KELVIN, energyUnit);
		String elupValueString = CassisDecimalFormat.EUP_FORMAT.format(eupValue);
		if (eupValue < 10E-1) {
			elupValueString = CassisDecimalFormat.TWO_DEC_EXP_FORMAT.format(eupValue);
		}
		map.put(EUP_VALUE, elupValueString);
		map.put(EUP_UNIT, energyUnit.getValString());

		map.put(AIJ_VALUE, CassisDecimalFormat.AIJ_FORMAT.format(line.getAij()));
		map.put(AIJ_UNIT, "");
		map.put(GUP_VALUE, String.valueOf(line.getGu()));
		map.put(GUP_UNIT, "");
		map.put(SHIFT_WAVE_LABEL, "Vlsr =");
		map.put(SHIFT_WAVE_VALUE, CassisDecimalFormat.TWO_DEC_FORMAT.format(vlsrPlot));
		map.put(SHIFT_WAVE_UNIT,  "km/s");

		if (line.getCitation() != null && !line.getCitation().isEmpty()) {
			map.put(CITATION_LABEL, VamdcDataBaseConnection.VAMDC_REQUEST_TOKEN+ "=");
			map.put(CITATION_VALUE, line.getCitation());

		}
		if (line.getOtherFreqs() != null &&  !line.getOtherFreqs().isEmpty()) {
			map.put(OTHER_WAVE, "Other frequencies proposed: " + line.getOtherFreqs());
		}
		return map;
	}

	/**
	 * Update the raw line String by the data in the map (with a [[key]] to value replacement).
	 *
	 * @param map The map with the data to replace as key and the replacement as value.
	 * @param rawLine The raw line String.
	 * @return The updated line.
	 */
	private static String replaceLineIdentificationValue(HashMap<String, String> map, String rawLine) {
		for (String key : map.keySet()) {
			rawLine = rawLine.replaceAll("\\[\\[" + key + "\\]\\]", Matcher.quoteReplacement(map.get(key)));
		}
		rawLine = rawLine.replaceAll("(\\[\\[)(.*?)(\\]\\])", "");
		return rawLine.trim();
	}

	/**
	 * Update the line identification from the user template and given parameters
	 * for an Other Species line.
	 *
	 * @param line The line
	 * @param wave The wave
	 * @param waveError The wave error
	 * @param energyUnit The energy unit
	 * @param vlsrPlot The VLSR of the plot
	 * @param xAxisCassis The XAxisCassis used
	 * @return the mapping
	 */
	public static void updateLine(LineDescription line, double wave, double waveError,
			UNIT energyUnit, double vlsrPlot, XAxisCassis xAxisCassis) {
		String rawLine = Software.getUserConfiguration().getOtherSpeciesLinesTemplate();
		updateLine(line, wave, waveError, energyUnit, vlsrPlot, xAxisCassis, rawLine);
	}

	/**
	 * Update the line identification from the given template and given parameters
	 * for an Other Species line.
	 *
	 * @param line The line
	 * @param wave The wave
	 * @param waveError The wave error
	 * @param energyUnit The energy unit
	 * @param vlsrPlot The VLSR of the plot
	 * @param xAxisCassis The XAxisCassis used
	 * @param rawLine The line description template
	 * @return the mapping
	 */
	public static void updateLine(LineDescription line, double wave,
			double waveError, UNIT energyUnit, double vlsrPlot,
			XAxisCassis xAxisCassis, String rawLine) {
		HashMap<String, String> map = getParameterValueMapping(line,
				wave, waveError, energyUnit, vlsrPlot, xAxisCassis);
		String replaceLineIdentificationValue = replaceLineIdentificationValue(map, rawLine);
		line.setIdentification(replaceLineIdentificationValue);
	}

}
