/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.otherspecies;

import java.util.List;
import java.util.concurrent.ExecutionException;

import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import eu.omp.irap.cassis.common.LineDescription;
import eu.omp.irap.cassis.common.axes.XAxisCassis;
import eu.omp.irap.cassis.properties.Software;

/**
 * Other species worker for Full Spectrum.
 */
public class SpeciesWorker extends SwingWorker<SpeciesResult, Void> {

	private SpeciesInterface speciesInterface;
	private SpeciesControl control;
	private double waveMinSignal;
	private double waveMaxSignal;
	private double waveMinImage;
	private double waveMaxImage;
	private XAxisCassis xAxis;
	private String commentSignal;
	private String commentImage;
	private boolean computeImage;


	/**
	 * Utility method to construct a species worker to compute only signal lines.
	 *
	 * @param speciesInterface The SpeciesInterface.
	 * @param control The SpeciesControl.
	 * @param waveMin The minimum wave value.
	 * @param waveMax The maximum wave value.
	 * @param xAxis The X axis.
	 * @param comment Comment to add.
	 * @return The SpeciesWorker created from the given parameters.
	 */
	public static SpeciesWorker getSignalSpeciesWorker(SpeciesInterface speciesInterface,
			SpeciesControl control, double waveMin, double waveMax,
			XAxisCassis xAxis, String comment) {
		return new SpeciesWorker(speciesInterface, control, waveMin, waveMax, xAxis,
				comment, Double.NaN, Double.NaN, null, false);
	}

	/**
	 * Utility method to construct a species worker to compute signal and image lines.
	 *
	 * @param speciesInterface The SpeciesInterface.
	 * @param control The SpeciesControl.
	 * @param waveMinSignal The minimum wave value for signal.
	 * @param waveMaxSignal The maximum wave value for signal.
	 * @param xAxis The X axis.
	 * @param commentSignal Comment to add for signal lines.
	 * @param waveMinImage The minimum wave value for image.
	 * @param waveMaxImage The maximum wave value for image.
	 * @param commentImage Comment to add for image lines.
	 * @return The SpeciesWorker created from the given parameters.
	 */
	public static SpeciesWorker getSignalImageSpeciesWorker(
			SpeciesInterface speciesInterface, SpeciesControl control,
			double waveMinSignal, double waveMaxSignal, XAxisCassis xAxis,
			String commentSignal, double waveMinImage, double waveMaxImage,
			String commentImage) {
		return new SpeciesWorker(speciesInterface, control, waveMinSignal,
				waveMaxSignal, xAxis, commentSignal, waveMinImage, waveMaxImage,
				commentImage, true);
	}

	/**
	 * Constructor.
	 *
	 * @param speciesInterface The SpeciesInterface.
	 * @param control The SpeciesControl.
	 * @param waveMinSignal The minimum wave value for signal.
	 * @param waveMaxSignal The maximum wave value for signal.
	 * @param xAxis The X axis.
	 * @param commentSignal Comment to add for signal lines.
	 * @param waveMinImage The minimum wave value for image.
	 * @param waveMaxImage The maximum wave value for image.
	 * @param commentImage Comment to add for image lines.
	 * @param computeImage true to compute images lines, false otherwise.
	 */
	private SpeciesWorker(SpeciesInterface speciesInterface, SpeciesControl control,
			double waveMinSignal, double waveMaxSignal, XAxisCassis xAxis,
			String commentSignal, double waveMinImage, double waveMaxImage,
			String commentImage, boolean computeImage) {
		super();
		this.speciesInterface = speciesInterface;
		this.control = control;
		this.waveMinSignal = waveMinSignal;
		this.waveMaxSignal = waveMaxSignal;
		this.xAxis = xAxis;
		this.commentSignal = commentSignal;
		this.waveMinImage = waveMinImage;
		this.waveMaxImage = waveMaxImage;
		this.commentImage = commentImage;
		this.computeImage = computeImage;
	}

	/**
	 * Compute the lines in a background thread (not EDT).
	 */
	@Override
	protected SpeciesResult doInBackground() throws Exception {
		List<LineDescription> signalLines = control.computeOtherSpecies(
				waveMinSignal, waveMaxSignal, xAxis, true, commentSignal);
		if (isCancelled()) return null;
		List<LineDescription> imageLines = null;
		if (computeImage) {
			imageLines = control.computeOtherSpecies(waveMinImage,
					waveMaxImage, xAxis, false, commentImage);
		}
		if (isCancelled()) return null;
		return new SpeciesResult(signalLines, imageLines);
	}

	/**
	 * Handle the result (lines) in the EDT.
	 */
	@Override
	protected void done() {
		if (isCancelled()) {
			speciesInterface.displayDataset();
		} else {
			try {
				SpeciesResult result = get();
				if (result.hasSignalLines() || result.hasImageLines()) {
					speciesInterface.displayOtherSpecies(result.getSignalLines(), result.getImageLines());
				} else if (!Software.getUserConfiguration().isAutoOtherSpeciesDisplayed() ||
						   !control.getModel().getCassisTableMoleculeModel().getSelectedMolecules().isEmpty()) {
					JOptionPane.showMessageDialog(speciesInterface.getView(),
							"No transition found. Check your thresholds and/or template if necessary.",
							"Other Species", JOptionPane.WARNING_MESSAGE);
				}
			} catch (InterruptedException | ExecutionException e) {
				// Ignore it.
			}
		}
		control.getView().setDone();
	}

	/**
	 * Stop the worker. Note: while it handle some case to clean up the operation,
	 *  it will not stop currently running database query or I/O.
	 */
	public void stop() {
		try {
			this.cancel(true);
		} catch (Exception e) {
			// Nothing to do, ignore it.
		}
		try {
			Thread.currentThread().interrupt();
		} catch (Exception e) {
			// Nothing to do, ignore it.
		}
	}
}

