/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.plot.simple.renderer;

import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

import org.jfree.chart.HashUtilities;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.event.RendererChangeEvent;
import org.jfree.chart.plot.CrosshairState;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.AbstractXYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRendererState;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleEdge;
import org.jfree.util.BooleanList;
import org.jfree.util.ObjectUtilities;
import org.jfree.util.PaintList;
import org.jfree.util.PublicCloneable;
import org.jfree.util.StrokeList;

import eu.omp.irap.cassis.gui.plot.simple.collection.SeriesCassisCollection;
import eu.omp.irap.cassis.gui.plot.simple.series.SeriesCassis;
import eu.omp.irap.cassis.gui.plot.simple.series.SpectrumSeriesCassis;
import eu.omp.irap.cassis.gui.plot.simple.util.DoubleList;
import eu.omp.irap.cassis.properties.Software;

/**
 * Renderer allowing to plot with dot and changing the size of the points by series.
 *
 * @author M. Boiziot
 */
public class XYDotBySeriesRenderer extends AbstractXYItemRenderer implements XYItemRenderer, PublicCloneable {

	private static final long serialVersionUID = 1644353906262512190L;
	private IntegerList seriesDotHeight;
	private IntegerList seriesDotWidth;

	/** The length of the cap at the end of the error bars. */
	private DoubleList capLengthList;

	/**
	 * A table of flags that control (per series) whether or not errors are
	 * visible.
	 */
	private BooleanList errorLinesVisible;

	/** The error paint list. */
	private PaintList errorPaintList;

	/** The stroke list. */
	private StrokeList errorStrokeList;


	/**
	 * Constructor.
	 */
	public XYDotBySeriesRenderer() {
		super();
		this.seriesDotHeight = new IntegerList();
		this.seriesDotWidth = new IntegerList();
		this.errorLinesVisible = new BooleanList();
		this.errorPaintList = new PaintList();
		this.errorStrokeList = new StrokeList();
		this.capLengthList = new DoubleList(Software.getUserConfiguration().getErrorCapLength());
	}

	/**
	 * Returns the flag used to control whether or not the error for a series
	 * are visible.
	 *
	 * @param series  the series index (zero-based).
	 *
	 * @return The flag. True if not found.
	 *
	 * @see #setErrorLinesVisible(int, Boolean)
	 */
	public Boolean getErrorLinesVisible(int series) {
		Boolean res = this.errorLinesVisible.getBoolean(series);
		if (res == null) {
			res = true;
		}
		return res;
	}

	/**
	 * Sets the 'error visible' flag for a series and sends a
	 * {@link RendererChangeEvent} to all registered listeners.
	 *
	 * @param series  the series index (zero-based).
	 * @param flag  the flag (<code>null</code> permitted).
	 *
	 * @see #getErrorLinesVisible(int)
	 */
	public void setErrorLinesVisible(int series, Boolean flag) {
		this.errorLinesVisible.setBoolean(series, flag);
		fireChangeEvent();
	}

	/**
	 * Returns the stroke used to draw the errors in a series.
	 *
	 * @param series  the series (zero-based index).
	 *
	 * @return The stroke (possibly <code>null</code>).
	 *
	 * @see #setErrorStroke(int, Stroke)
	 */
	public Stroke getErrorStroke(int series) {
		return this.errorStrokeList.getStroke(series);
	}

	/**
	 * Sets the stroke used for the error in series and sends a
	 * {@link RendererChangeEvent} to all registered listeners.
	 *
	 * @param series  the series index (zero-based).
	 * @param stroke  the stroke (<code>null</code> permitted).
	 *
	 * @see #getErrorStroke(int)
	 */
	public void setErrorStroke(int series, Stroke stroke) {
		this.errorStrokeList.setStroke(series, stroke);
		fireChangeEvent();
	}


	/**
	 * Returns the paint defined for the specified error of the series (possibly
	 * <code>null</code>).
	 *
	 * @param series  the series index.
	 *
	 * @return The paint (possibly <code>null</code>).
	 */
	public Paint getErrorPaint(int series) {
		return this.errorPaintList.getPaint(series);
	}

	/**
	 * Sets the paint used for the error for the specified series, and
	 * sends a {@link RendererChangeEvent} to all registered listeners.
	 *
	 * @param series  the series index.
	 * @param paint  the paint (<code>null</code> permitted).
	 */
	public void setErrorPaint(int series, Paint paint) {
		this.errorPaintList.setPaint(series, paint);
		fireChangeEvent();
	}

	/**
	 * Returns the cap length used to draw the errors in a series.
	 *
	 * @param series  the series (zero-based index).
	 * @return The cap length
	 */
	public double getCapLength(int series) {
		return this.capLengthList.getDouble(series);
	}

	/**
	 * Sets the cap length used for the error in series and sends a
	 * {@link RendererChangeEvent} to all registered listeners.
	 *
	 * @param series  the series index (zero-based).
	 * @param value the cap length value to set.
	 */
	public void setCapLength(int series, double value) {
		this.capLengthList.setDouble(series, value);
		fireChangeEvent();
	}

	/**
	 * Draws the visual representation of a single data item.
	 *
	 * @param g2 the graphics device.
	 * @param state the renderer state.
	 * @param dataArea the area within which the data is being drawn.
	 * @param info collects information about the drawing.
	 * @param plot the plot (can be used to obtain standard color information etc).
	 * @param domainAxis the domain (horizontal) axis.
	 * @param rangeAxis the range (vertical) axis.
	 * @param dataset the dataset.
	 * @param series the series index (zero-based).
	 * @param item the item index (zero-based).
	 * @param crosshairState crosshair information for the plot (<code>null</code> permitted).
	 * @param pass the pass index.
	 */
	@Override
	public void drawItem(Graphics2D g2, XYItemRendererState state, Rectangle2D dataArea,
			PlotRenderingInfo info, XYPlot plot, ValueAxis domainAxis,
			ValueAxis rangeAxis, XYDataset dataset, int series, int item,
			CrosshairState crosshairState, int pass) {
		if (getErrorLinesVisible(series)) {
			drawItemErrorCassis(g2, state, dataArea, info, plot, domainAxis,
					rangeAxis, dataset, series, item, crosshairState, pass);
		}
		drawItemDot(g2, state, dataArea, info, plot, domainAxis, rangeAxis,
				dataset, series, item, crosshairState, pass);
	}

	/**
	 * Tests this renderer for equality with an arbitrary object. This method
	 * returns <code>true</code> if and only if:
	 *
	 * <ul>
	 * <li><code>obj</code> is not <code>null</code>;</li>
	 * <li><code>obj</code> is an instance of <code>XYDotRenderer</code>;</li>
	 * <li>both renderers have the same attribute values.
	 * </ul>
	 *
	 * @param obj the object (<code>null</code> permitted).
	 *
	 * @return A boolean.
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (!(obj instanceof XYDotBySeriesRenderer)) {
			return false;
		}
		XYDotBySeriesRenderer that = (XYDotBySeriesRenderer) obj;
		if (!this.seriesDotWidth.equals(that.seriesDotWidth)) {
			return false;
		}
		if (!this.seriesDotHeight.equals(that.seriesDotHeight)) {
			return false;
		}
		if (!this.capLengthList.equals(that.capLengthList)) {
			return false;
		}
		if (!this.errorLinesVisible.equals(that.errorLinesVisible)) {
			return false;
		}
		if (!ObjectUtilities.equal(this.errorPaintList, that.errorPaintList)) {
			return false;
		}
		if (!ObjectUtilities.equal(this.errorStrokeList, that.errorStrokeList)) {
			return false;
		}
		return super.equals(obj);
	}

	/**
	 * Returns a clone of the renderer.
	 *
	 * @return A clone.
	 *
	 * @throws CloneNotSupportedException if the renderer cannot be cloned.
	 */
	@Override
	public Object clone() throws CloneNotSupportedException {
		XYDotBySeriesRenderer s = (XYDotBySeriesRenderer) super.clone();
		s.seriesDotHeight = (IntegerList) seriesDotHeight.clone();
		s.seriesDotWidth = (IntegerList) seriesDotWidth.clone();
		s.capLengthList = (DoubleList) this.capLengthList.clone();
		s.errorLinesVisible = (BooleanList) this.errorLinesVisible.clone();
		s.errorPaintList = (PaintList) this.errorPaintList.clone();
		s.errorStrokeList = (StrokeList) this.errorStrokeList.clone();
		return s;
	}

	/**
	 * Change the height of dot.
	 *
	 * @param series The series for which we want to change the size of the dot.
	 * @param dotHeight The dot height to set.
	 */
	public void setSeriesDotHeight(int series, int dotHeight) {
		setSeriesDotHeight(series, Integer.valueOf(dotHeight));
	}

	/**
	 * Change the height of dot.
	 *
	 * @param series The series for which we want to change the size of the dot.
	 * @param dotHeight The dot height to set.
	 */
	public void setSeriesDotHeight(int series, Integer dotHeight) {
		this.seriesDotHeight.setInteger(series, dotHeight);
		fireChangeEvent();
	}

	/**
	 * Change the width of dot.
	 *
	 * @param series The series for which we want to change the size of the dot.
	 * @param dotWidth The dot width to set.
	 */
	public void setSeriesDotWidth(int series, int dotWidth) {
		setSeriesDotWidth(series, Integer.valueOf(dotWidth));
	}

	/**
	 * Change the height of dot.
	 *
	 * @param series The series for which we want to change the size of the dot.
	 * @param dotWidth The dot width to set.
	 */
	public void setSeriesDotWidth(int series, Integer dotWidth) {
		this.seriesDotWidth.setInteger(series, dotWidth);
		fireChangeEvent();
	}

	/**
	 * Return the dot height for the given series index.
	 *
	 * @param series The index of the series.
	 * @return The dot height.
	 */
	public Integer getSeriesDotHeight(int series) {
		return this.seriesDotHeight.getInteger(series);
	}

	/**
	 * Return the dot width for the given series index.
	 *
	 * @param series The index of the series.
	 * @return The dot width.
	 */
	public Integer getSeriesDotWidth(int series) {
		return this.seriesDotWidth.getInteger(series);
	}

	/**
	 * Create a Hash Code for the object.
	 *
	 * @return a hash code for the object.
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result
				+ ((seriesDotHeight == null) ? 0 : seriesDotHeight.hashCode());
		result = prime * result
				+ ((seriesDotWidth == null) ? 0 : seriesDotWidth.hashCode());
		result = HashUtilities.hashCode(result, capLengthList);
		result = HashUtilities.hashCode(result, errorLinesVisible);
		result = HashUtilities.hashCode(result, errorPaintList);
		result = HashUtilities.hashCode(result, errorStrokeList);
		return result;
	}

	/**
	 * Draws the visual representation of a single data item.
	 *
	 * @param g2 the graphics device.
	 * @param state the renderer state.
	 * @param dataArea the area within which the data is being drawn.
	 * @param info collects information about the drawing.
	 * @param plot the plot (can be used to obtain standard color information etc).
	 * @param domainAxis the domain (horizontal) axis.
	 * @param rangeAxis the range (vertical) axis.
	 * @param dataset the dataset.
	 * @param series the series index (zero-based).
	 * @param item the item index (zero-based).
	 * @param crosshairState crosshair information for the plot (<code>null</code> permitted).
	 * @param pass the pass index.
	 */
	public void drawItemDot(Graphics2D g2, XYItemRendererState state, Rectangle2D dataArea,
			PlotRenderingInfo info, XYPlot plot, ValueAxis domainAxis,
			ValueAxis rangeAxis, XYDataset dataset, int series, int item,
			CrosshairState crosshairState, int pass) {

		// do nothing if item is not visible
		if (!getItemVisible(series, item)) {
			return;
		}

		// get the data point...
		double x = dataset.getXValue(series, item);
		double y = dataset.getYValue(series, item);
		int currentSeriesDotWidth = seriesDotWidth.getInteger(series);
		int currentSeriesDotHeight = seriesDotHeight.getInteger(series);
		double adjx = (currentSeriesDotWidth - 1) / 2.0;
		double adjy = (currentSeriesDotHeight - 1) / 2.0;
		if (!Double.isNaN(y)) {
			RectangleEdge xAxisLocation = plot.getDomainAxisEdge();
			RectangleEdge yAxisLocation = plot.getRangeAxisEdge();
			double transX = domainAxis.valueToJava2D(x, dataArea, xAxisLocation) - adjx;
			double transY = rangeAxis.valueToJava2D(y, dataArea, yAxisLocation) - adjy;

			g2.setPaint(getItemPaint(series, item));
			PlotOrientation orientation = plot.getOrientation();
			if (orientation == PlotOrientation.HORIZONTAL) {
				g2.fillRect((int) transY, (int) transX, currentSeriesDotHeight,
						currentSeriesDotWidth);
			} else if (orientation == PlotOrientation.VERTICAL) {
				g2.fillRect((int) transX, (int) transY, currentSeriesDotWidth,
						currentSeriesDotHeight);
			}

			int domainAxisIndex = plot.getDomainAxisIndex(domainAxis);
			int rangeAxisIndex = plot.getRangeAxisIndex(rangeAxis);
			updateCrosshairValues(crosshairState, x, y, domainAxisIndex, rangeAxisIndex,
					transX, transY, orientation);
		}
	}

	/**
	 * Draws the visual representation for one data item.
	 *
	 * @param g2  the graphics output target.
	 * @param state  the renderer state.
	 * @param dataArea  the data area.
	 * @param info  the plot rendering info.
	 * @param plot  the plot.
	 * @param domainAxis  the domain axis.
	 * @param rangeAxis  the range axis.
	 * @param dataset  the dataset.
	 * @param series  the series index.
	 * @param item  the item index.
	 * @param crosshairState  the crosshair state.
	 * @param pass  the pass index.
	 */
	public void drawItemErrorCassis(Graphics2D g2, XYItemRendererState state,
			Rectangle2D dataArea, PlotRenderingInfo info, XYPlot plot,
			ValueAxis domainAxis, ValueAxis rangeAxis, XYDataset dataset,
			int series, int item, CrosshairState crosshairState, int pass) {
		if (pass == 0 && dataset instanceof SeriesCassisCollection
				&& getItemVisible(series, item)) {
			SeriesCassisCollection col = (SeriesCassisCollection) dataset;
			PlotOrientation orientation = plot.getOrientation();
			boolean hasYError = false;
			SeriesCassis seriesCassis = col.getSeries(series);
			if (seriesCassis instanceof SpectrumSeriesCassis) {
				hasYError = ((SpectrumSeriesCassis)seriesCassis).hasYError();
			}
			if (hasYError) {
				double y0 = col.getErrorYLow(series, item);
				double y1 = col.getErrorYHigh(series, item);
				double x = col.getErrorX(series, item);
				if (!Double.isNaN(y0) && !Double.isNaN(y1) && !Double.isNaN(x)) {
					RectangleEdge edge = plot.getRangeAxisEdge();
					double yy0 = rangeAxis.valueToJava2D(y0, dataArea, edge);
					double yy1 = rangeAxis.valueToJava2D(y1, dataArea, edge);
					double xx = domainAxis.valueToJava2D(x, dataArea,
							plot.getDomainAxisEdge());
					Line2D line;
					Line2D cap1;
					Line2D cap2;
					double adj = getCapLength(series) / 2.0;
					if (orientation == PlotOrientation.VERTICAL) {
						line = new Line2D.Double(xx, yy0, xx, yy1);
						cap1 = new Line2D.Double(xx - adj, yy0, xx + adj, yy0);
						cap2 = new Line2D.Double(xx - adj, yy1, xx + adj, yy1);
					}
					else {  // PlotOrientation.HORIZONTAL
						line = new Line2D.Double(yy0, xx, yy1, xx);
						cap1 = new Line2D.Double(yy0, xx - adj, yy0, xx + adj);
						cap2 = new Line2D.Double(yy1, xx - adj, yy1, xx + adj);
					}
					Paint errorPaint = getErrorPaint(series);
					if (errorPaint != null) {
						g2.setPaint(errorPaint);
					}
					else {
						g2.setPaint(getItemPaint(series, item));
					}

					Stroke errorStroke = getErrorStroke(series);
					if (errorStroke != null) {
						g2.setStroke(errorStroke);
					}
					else {
						g2.setStroke(getItemStroke(series, item));
					}
					g2.draw(line);
					g2.draw(cap1);
					g2.draw(cap2);
				}
			}
		}
		drawItemLineAndShapeRenderer(g2, state, dataArea, info, plot, domainAxis, rangeAxis,
				dataset, series, item, crosshairState, pass);
	}

	/**
	 * Draws the visual representation of a single data item.
	 *
	 * @param g2  the graphics device.
	 * @param state  the renderer state.
	 * @param dataArea  the area within which the data is being drawn.
	 * @param info  collects information about the drawing.
	 * @param plot  the plot (can be used to obtain standard color
	 *			  information etc).
	 * @param domainAxis  the domain axis.
	 * @param rangeAxis  the range axis.
	 * @param dataset  the dataset.
	 * @param series  the series index (zero-based).
	 * @param item  the item index (zero-based).
	 * @param crosshairState  crosshair information for the plot
	 *						(<code>null</code> permitted).
	 * @param pass  the pass index.
	 */
	public void drawItemLineAndShapeRenderer(Graphics2D g2, XYItemRendererState state,
			Rectangle2D dataArea, PlotRenderingInfo info, XYPlot plot,
			ValueAxis domainAxis, ValueAxis rangeAxis, XYDataset dataset,
			int series, int item, CrosshairState crosshairState, int pass) {

		// do nothing if item is not visible
		if (!getItemVisible(series, item)) {
			return;
		}

		if (isItemPass(pass)) {

			// setup for collecting optional entity info...
			EntityCollection entities = null;
			if (info != null && info.getOwner() != null) {
				entities = info.getOwner().getEntityCollection();
			}

			drawSecondaryPass(g2, plot, dataset, pass, series, item,
					domainAxis, dataArea, rangeAxis, crosshairState, entities);
		}
	}

	/**
	 * Returns <code>true</code> if the specified pass is the one for drawing
	 * items.
	 *
	 * @param pass  the pass.
	 *
	 * @return A boolean.
	 */
	protected boolean isItemPass(int pass) {
		return pass == 1;
	}

	// FROM XYLineAndShapeRenderer
	/**
	 * Draws the item shapes and adds chart entities (second pass). This method
	 * draws the shapes which mark the item positions. If <code>entities</code>
	 * is not <code>null</code> it will be populated with entity information
	 * for points that fall within the data area.
	 *
	 * @param g2  the graphics device.
	 * @param plot  the plot (can be used to obtain standard color
	 *			  information etc).
	 * @param domainAxis  the domain axis.
	 * @param dataArea  the area within which the data is being drawn.
	 * @param rangeAxis  the range axis.
	 * @param dataset  the dataset.
	 * @param pass  the pass.
	 * @param series  the series index (zero-based).
	 * @param item  the item index (zero-based).
	 * @param crosshairState  the crosshair state.
	 * @param entities the entity collection.
	 */
	protected void drawSecondaryPass(Graphics2D g2, XYPlot plot,
			XYDataset dataset, int pass, int series, int item,
			ValueAxis domainAxis, Rectangle2D dataArea, ValueAxis rangeAxis,
			CrosshairState crosshairState, EntityCollection entities) {

		Shape entityArea = null;

		// get the data point...
		double x1 = dataset.getXValue(series, item);
		double y1 = dataset.getYValue(series, item);
		if (Double.isNaN(y1) || Double.isNaN(x1)) {
			return;
		}

		PlotOrientation orientation = plot.getOrientation();
		RectangleEdge xAxisLocation = plot.getDomainAxisEdge();
		RectangleEdge yAxisLocation = plot.getRangeAxisEdge();
		double transX1 = domainAxis.valueToJava2D(x1, dataArea, xAxisLocation);
		double transY1 = rangeAxis.valueToJava2D(y1, dataArea, yAxisLocation);

		double xx = transX1;
		double yy = transY1;
		if (orientation == PlotOrientation.HORIZONTAL) {
			xx = transY1;
			yy = transX1;
		}

		// draw the item label if there is one...
		if (isItemLabelVisible(series, item)) {
			drawItemLabel(g2, orientation, dataset, series, item, xx, yy,
					(y1 < 0.0));
		}

		int domainAxisIndex = plot.getDomainAxisIndex(domainAxis);
		int rangeAxisIndex = plot.getRangeAxisIndex(rangeAxis);
		updateCrosshairValues(crosshairState, x1, y1, domainAxisIndex,
				rangeAxisIndex, transX1, transY1, orientation);

		// add an entity for the item, but only if it falls within the data
		// area...
		if (entities != null && isPointInRect(dataArea, xx, yy)) {
			addEntity(entities, entityArea, dataset, series, item, xx, yy);
		}
	}

}
