/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.parameters;

import java.util.List;

import eu.omp.irap.cassis.gui.model.parameter.rotationaltuning.TYPE_TEMPERATURE;
import eu.omp.irap.cassis.gui.model.rotationaldiagram.RotationalMoleculeConf;


/**
 * Class describing the input for the computing of RotationalDiagram.
 *
 * @author M. Boiziot
 */
public class RotationalDiagramInput {

	private String pathFileSource;
	private List<RotationalMolecule> listRotMol;
	private List<RotationalMoleculeConf> listMolConf;
	private TYPE_TEMPERATURE typeTemp;
	private boolean taToTmbSelected;


	/**
	 * Constructor.
	 *
	 * @param pathFileSource The path of the file used as source.
	 *
	 * @param listRotMol The list of {@link RotationalMolecule}.
	 * @param listMolConf The list of {@link RotationalMoleculeConf}.
	 * @param typeTemp The type of temperature of the data.
	 * @param taToTmbSelected If taToTmb has been selected.
	 */
	public RotationalDiagramInput(String pathFileSource,
			List<RotationalMolecule> listRotMol,
			List<RotationalMoleculeConf> listMolConf,
			TYPE_TEMPERATURE typeTemp, boolean taToTmbSelected) {
		this.pathFileSource = pathFileSource;
		this.listRotMol = listRotMol;
		this.listMolConf = listMolConf;
		this.typeTemp = typeTemp;
		this.taToTmbSelected = taToTmbSelected;
	}

	/**
	 * Return the path of the file used as source.
	 *
	 * @return The path of the file used as source.
	 */
	public String getPathFileSource() {
		return pathFileSource;
	}

	/**
	 * Return the List of {@link RotationalMolecule}.
	 *
	 * @return The list of {@link RotationalMolecule}.
	 */
	public List<RotationalMolecule> getListRotMol() {
		return listRotMol;
	}

	/**
	 * Return the List of {@link RotationalMoleculeConf}.
	 *
	 * @return The list of {@link RotationalMoleculeConf}.
	 */
	public List<RotationalMoleculeConf> getListMolConf() {
		return listMolConf;
	}

	/**
	 * Return the {@link RotationalMoleculeConf} associated the provided tag.
	 *
	 * @param speciesId The tag of the molecule for wich we want the {@link RotationalMoleculeConf}.
	 * @return The {@link RotationalMoleculeConf} associated the provided tag, or null if not found.
	 */
	public RotationalMoleculeConf getMolConf(String speciesId) {
		for (RotationalMoleculeConf molConf : listMolConf) {
			if (molConf.getSpeciesId().equals(speciesId)) {
				return molConf;
			}
		}
		return null;
	}

	/**
	 * Return if TaToTmb is selected.
	 *
	 * @return true if TaToTmb is selected, false otherwise.
	 */
	public boolean isTaToTmbSelected() {
		return taToTmbSelected;
	}

	/**
	 * Return the temperature type of the data.
	 *
	 * @return the temperature type of the data.
	 */
	public TYPE_TEMPERATURE getTypeTemperature() {
		return typeTemp;
	}
}
